//////////////////////////////////////////////////////////////////////////////////////////////////
// VERSION 4.0 
//
// File: camera.h
// 
// Author: Frank Luna (C) All Rights Reserved + Booch =))
//
// System: AMD Athlon 1800+ XP, 512 DDR, Geforce 3, Windows XP, MSVC++ 7.0 
//
// Desc: Defines a camera's position and orientation.
//         
//////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef __cameraH__
#define __cameraH__

#include <d3d9.h>
#include <d3dx9.h>
#pragma comment(lib, "d3d9.lib")
#pragma comment(lib, "d3dx9.lib")

class Camera
{
public:
	enum CameraType { LANDOBJECT, AIRCRAFT };

	Camera();
	Camera(CameraType cameraType);
	~Camera();

	void strafe(float units); // left/right
	void fly(float units);    // up/down
	void walk(float units);   // forward/backward
	
	void pitch(float angle); // rotate on right vector
	void yaw(float angle);   // rotate on up vector
	void roll(float angle);  // rotate on look vector

	void getViewMatrix(D3DXMATRIX* V); 
	void getProjMatrix(D3DXMATRIX* Proj);
	void getViewProjMatrix(D3DXMATRIX* ViewProj);
	void getSkyBoxMatrix(D3DXMATRIX* SkyBox, IDirect3DDevice9* device);
	void setCameraType(CameraType cameraType); 
	void getPosition(D3DXVECTOR3* pos); 
	void setPosition(D3DXVECTOR3* pos); 

	void getRight(D3DXVECTOR3* right);
	void getUp(D3DXVECTOR3* up);
	void getLook(D3DXVECTOR3* look);
	//  
	void setRight(D3DXVECTOR3* right);
	void setUp(D3DXVECTOR3* up);
	void setLook(D3DXVECTOR3* look);

	void ReadInput( float timeDelta );
	//      ,    =))
	void useCamera( IDirect3DDevice9*  device );
	//  ( _Proj),  0,     
	void setProjParam(IDirect3DDevice9*  device, float fov, float asp, float zN, float zF);
	void focusOn( D3DXVECTOR3* point, float speed );

	//  (0-2Pi)   2D( ),  X,
	//   
	//Y
	//^		
	//|
	//.------------>X
	//X1,Y1			.X2,Y2
	//
	float Ugol2D(float X1, float Y1, float X2, float Y2);
	//  1.0,-1.0  0.0  2D   2D  Xt,Yt
	float UgTurnVec2D(float Xr, float Yr, float dx, float dy,float Xt, float Yt);

private:
	D3DXMATRIX  _Proj, _matUnitProj;	// matUnitProj -    
	float		_fovAngle, _aspect, _zNear, _zFar;
	CameraType  _cameraType;
	D3DXVECTOR3 _right;
	D3DXVECTOR3 _up;
	D3DXVECTOR3 _look;
	D3DXVECTOR3 _pos;
};
#endif // __cameraH__