#include "stdafx.h"
#include "sky_model.h"

#define safe_delete_array(p) { if(p) { delete[] (p);   (p)=0; } }

struct Vertex
	{
		Vertex(){}
		Vertex(
			float x, float y, float z
			  )
		{
			_x  = x;  _y  = y;  _z  = z;
		}
		float _x, _y, _z;
	};
#define FVF_VERTEX (D3DFVF_XYZ)

cSkyModel::cSkyModel(IDirect3DDevice9* device, Camera* TheCamera)
{
	Device = device;
	D3DXMatrixIdentity(&_world);
	_SkyBoxEffect = 0;
	_WorldViewProjHandle = 0;
	_TexHandle = 0;
	_SkyBoxTechHandle = 0;
	D3DXMatrixIdentity(&_WorldViewProj);
	_TheCamera = TheCamera;
	// -   
	m_width=(D3DX_DEFAULT);
	m_height=(D3DX_DEFAULT);
	m_mipLevels=(D3DX_DEFAULT);
	m_d3dUsage=(0);
	m_d3dFormat=(D3DFMT_UNKNOWN);
	m_d3dPool=(D3DPOOL_MANAGED);
	m_d3dFilter=(D3DX_DEFAULT);
	m_d3dMipFilter=(D3DX_DEFAULT);
	m_d3dColorKey=(0);
	m_pPalette=(0);
	m_pCubeTexture=(0);
	memset(&m_d3dImageInfo, 0, sizeof(m_d3dImageInfo));
}

cSkyModel::~cSkyModel()
{
	if(_vb){_vb->Release(); _vb = 0;}
	if(_ib){_ib->Release(); _ib = 0;}
	if(m_pCubeTexture) m_pCubeTexture->Release();

//	Device->Release();	// -   g  ,   
}

bool cSkyModel::loadCubeTexture(char *filename)
{
	//
	// 
	//
	//    
	HRESULT hr=0;
	hr = D3DXGetImageInfoFromFile((LPCSTR)filename, &m_d3dImageInfo);
	// 
	// we convert paletized bitmaps (D3DX_P8)
	if (m_d3dImageInfo.Format == D3DFMT_P8)
	{
		m_d3dImageInfo.Format = D3DFMT_A8;
	}
	if (SUCCEEDED(hr))
	{
		// copy the image information to our local members
		m_width = m_d3dImageInfo.Width;
		m_height = m_d3dImageInfo.Height;
		m_mipLevels = m_d3dImageInfo.MipLevels;
		m_d3dFormat = m_d3dImageInfo.Format;
		// attempt to load the real texture
		if (m_d3dImageInfo.ResourceType == D3DRTYPE_CUBETEXTURE)
		{
			
			HRESULT hr = D3DXCheckCubeTextureRequirements(Device,
										(UINT*)&m_width,
										(UINT*)&m_mipLevels,
										m_d3dUsage,
										&m_d3dFormat,
										m_d3dPool);		
			if (SUCCEEDED(hr))
			{
				// check for paletized formats
				safe_delete_array(m_pPalette);
				if (m_d3dFormat == D3DFMT_P8)
				{
					m_pPalette = new PALETTEENTRY[256];
				}
			}
			 // attempt to load the texture
			hr = D3DXCreateCubeTextureFromFileEx(	Device, 
													(LPCSTR)filename,
													m_width,
													m_mipLevels,
													m_d3dUsage,
													m_d3dFormat,
													m_d3dPool,
													m_d3dFilter,
													m_d3dMipFilter,
													m_d3dColorKey,
													&m_d3dImageInfo,
													m_pPalette,
													&m_pCubeTexture);
		}
		else //if (m_d3dImageInfo.ResourceType == D3DRTYPE_TEXTURE)
		{
			::MessageBox(0, "SkyBox:\nTexture file is NOT CUBETEXTURE -> load FAILED", 0, 0);
			safe_delete_array(m_pPalette);
            return false;
		}
		
	} // if (SUCCEEDED(hr))
	else
	{
		::MessageBox(0, "SkyBox:\nD3DXCreateCubeTextureFromFileEx() -> FAILED \nTexture may does not exist", 0, 0);
		safe_delete_array(m_pPalette);
        return false;
	}
	return true;
}

bool cSkyModel::CreateCube( char *Texturename  )
{
	Device->CreateVertexBuffer(
		24 * sizeof(Vertex), 
		D3DUSAGE_WRITEONLY,
		FVF_VERTEX,
		D3DPOOL_MANAGED,
		&_vb,
		0);

	Vertex* v;
	float W=1.0f, H=1.0f, L=1.0f;
	_vb->Lock(0, 0, (void**)&v, 0);

	// build box

	// fill in the front face vertex data
	v[0] = Vertex( -W, -H, -L);
	v[1] = Vertex( -W,  H, -L);
	v[2] = Vertex(  W,  H, -L);
	v[3] = Vertex(  W, -H, -L);

	// fill in the back face vertex data
	v[4] = Vertex( -W, -H,  L);
	v[5] = Vertex(  W, -H,  L);
	v[6] = Vertex(  W,  H,  L);
	v[7] = Vertex( -W,  H,  L);

	// fill in the top face vertex data
	v[8]  = Vertex( -W,  H, -L);
	v[9]  = Vertex( -W,  H,  L);
	v[10] = Vertex(  W,  H,  L);
	v[11] = Vertex(  W,  H, -L);

	// fill in the bottom face vertex data
	v[12] = Vertex( -W, -H, -L);
	v[13] = Vertex(  W, -H, -L);
	v[14] = Vertex(  W, -H,  L);
	v[15] = Vertex( -W, -H,  L);

	// fill in the left face vertex data
	v[16] = Vertex( -W, -H,  L);
	v[17] = Vertex( -W,  H,  L);
	v[18] = Vertex( -W,  H, -L);
	v[19] = Vertex( -W, -H, -L);

	// fill in the right face vertex data
	v[20] = Vertex(  W, -H, -L);
	v[21] = Vertex(  W,  H, -L);
	v[22] = Vertex(  W,  H,  L);
	v[23] = Vertex(  W, -H,  L);

	_vb->Unlock();

	Device->CreateIndexBuffer(
		36 * sizeof(WORD),
		D3DUSAGE_WRITEONLY,
		D3DFMT_INDEX16,
		D3DPOOL_MANAGED,
		&_ib,
		0);

	WORD* i = 0;
	_ib->Lock(0, 0, (void**)&i, 0);

	// fill in the front face index data
	i[0] = 0; i[1] = 1; i[2] = 2;
	i[3] = 0; i[4] = 2; i[5] = 3;

	// fill in the back face index data
	i[6] = 4; i[7]  = 5; i[8]  = 6;
	i[9] = 4; i[10] = 6; i[11] = 7;

	// fill in the top face index data
	i[12] = 8; i[13] =  9; i[14] = 10;
	i[15] = 8; i[16] = 10; i[17] = 11;

	// fill in the bottom face index data
	i[18] = 12; i[19] = 13; i[20] = 14;
	i[21] = 12; i[22] = 14; i[23] = 15;

	// fill in the left face index data
	i[24] = 16; i[25] = 17; i[26] = 18;
	i[27] = 16; i[28] = 18; i[29] = 19;

	// fill in the right face index data
	i[30] = 20; i[31] = 21; i[32] = 22;
	i[33] = 20; i[34] = 22; i[35] = 23;

	_ib->Unlock();


//////////////////////////////////////////////////////////////////////	
	//
    //  
    //
    HRESULT hr = 0;
	ID3DXBuffer* errorBuffer = 0;
    hr = D3DXCreateEffectFromFile(
           Device,           //  
           "simple_skybox.fx",  //   
           0,                //   
           0,                //   ID3DXInclude
           D3DXSHADER_DEBUG, //  
           0,                //    
           &_SkyBoxEffect,   //     
           &errorBuffer);    //    

    //     
    if(errorBuffer)
    {
         ::MessageBox(0, (char*)errorBuffer->GetBufferPointer(), 0, 0);
		 errorBuffer->Release();
    }

    if(FAILED(hr))
    {
          ::MessageBox(0, "D3DXCreateEffectFromFile() - FAILED", 0, 0);
          return false;
    }

    //
    //     
    //
    _SkyBoxTechHandle  = _SkyBoxEffect->GetTechniqueByName("BasicCubeMap");
	_WorldViewProjHandle = _SkyBoxEffect->GetParameterByName(0, "mWorldViewProj");
    _TexHandle         = _SkyBoxEffect->GetParameterByName(0, "tex0");

    //
    //   
    //
    // 
    _SkyBoxEffect->SetMatrix(_WorldViewProjHandle, &_WorldViewProj);
    
	if(loadCubeTexture(Texturename))
		_SkyBoxEffect->SetTexture(_TexHandle, m_pCubeTexture); 
	else return false;


	return true;
}


bool cSkyModel::render(bool wireframe)
{
	
	Device->SetTransform(D3DTS_WORLD, &_world);
	
	//   
	_SkyBoxEffect->SetTechnique(_SkyBoxTechHandle);
    UINT numPasses = 0;
    
	_SkyBoxEffect->Begin(&numPasses, 0); // , 
	
	//   
	_TheCamera->getSkyBoxMatrix(&_WorldViewProj, Device);
	_SkyBoxEffect->SetMatrix(_WorldViewProjHandle, &_WorldViewProj);
	_SkyBoxEffect->SetTexture(_TexHandle, m_pCubeTexture);
    
	for(int i = 0; i < numPasses; i++)
    {

		_SkyBoxEffect->BeginPass(i);	// , 
		 
		Device->SetStreamSource(0, _vb, 0, sizeof(Vertex));
		Device->SetIndices(_ib);
		Device->SetFVF(FVF_VERTEX);
		Device->DrawIndexedPrimitive(
			D3DPT_TRIANGLELIST, 
			0,                  
			0,                  
			24,
			0,
			12);  

	}
    _SkyBoxEffect->End();			//  
	
	
	D3DXMATRIX I;
	D3DXMatrixIdentity(&I);
	Device->SetTransform(D3DTS_WORLD, &I);

	return true;
}